#!/bin/python3

import argparse
import tempfile
import os
import shutil
import subprocess

DEFAULT_PACK_OUTPUT_DIR = "./Packs"
DEFAULT_OUTER_PACK_SOURCES = "./Outer"
DEFAULT_INNER_PACK_SOURCES = "./Inner"
# directory within the (outer) pack holding the "inner" pack
CONTENT_DIR = "Content"
ENCRYPT_FILES_PASSWORD = "OutdatedVersion"
ADDITIONAL_PACK_SIZE_FILE = "additional.pack.size.kb"
REDUCED_PACK_SIZE_FILE = "reduced.pack.size.kb"
ENCRYPT_FILES_FILE = "encrypt.files"
RESTRICTED_FILES = [ADDITIONAL_PACK_SIZE_FILE,
                    REDUCED_PACK_SIZE_FILE, ENCRYPT_FILES_FILE, CONTENT_DIR]

parser = argparse.ArgumentParser(
    description='Create .pack files to use on V7.2+ platforms.')
parser.add_argument('game', help='Name of the game package')
parser.add_argument('--save-as', default="",
                    help="The output file name (defaults to " + DEFAULT_PACK_OUTPUT_DIR + "/[game].pack)")
parser.add_argument('--in-dir', default="",
                    help="The directory containing the pack.info (outer pack source dir - defaults to "+DEFAULT_OUTER_PACK_SOURCES+"/[game])")
parser.add_argument('--content', default="",
                    help="The directory containing the content for this pack (inner pack source dir - defaults to "+DEFAULT_INNER_PACK_SOURCES+"/[game])")
parser.add_argument('--extra-content', nargs="*",
                    help="List of extra files to include in the content directory of the pack")
parser.add_argument('--verbosity', default="1",
                    help="Sets the logging verbosity")

args = parser.parse_args()

packSource = args.in_dir if len(args.in_dir) > 0 else os.path.join(
    DEFAULT_OUTER_PACK_SOURCES, args.game)
packSource = os.path.abspath(packSource)
packContentSource = args.content if len(args.content) > 0 else os.path.join(
    DEFAULT_INNER_PACK_SOURCES, args.game)
packContentSource = os.path.abspath(packContentSource)
packSaveFilename = args.save_as if len(args.save_as) > 0 else os.path.join(
    DEFAULT_PACK_OUTPUT_DIR, args.game + ".pack")
packSaveFilename = os.path.abspath(packSaveFilename)
verbosity = int(args.verbosity)


def copy_recursive(path, outDir, excludeList=[], folderContentOnly=False):
    if path in excludeList:
        return

    copyDest = os.path.join(outDir, os.path.basename(path))
    if os.path.isfile(path):
        if verbosity > 3:
            print("Copying file %s..." % (path))
        shutil.copy(path, copyDest)
    elif os.path.isdir(path):
        if verbosity > 3:
            print("Copying directory %s..." % (path))
        if folderContentOnly:
            copyDest = outDir
        else:
            os.mkdir(copyDest)
        for filename in os.listdir(path):
            copy_recursive(os.path.join(path, filename), copyDest)


def read_lines(filename):
    with open(filename, 'r') as file:
        lines = file.readlines()
    return lines


packageDir = tempfile.mkdtemp("_"+args.game, "pack_")
if verbosity > 0:
    print("Creating new pack file...")

contentDir = os.path.join(packageDir, CONTENT_DIR)
os.mkdir(contentDir)
copy_recursive(packContentSource, contentDir, [], True)
if args.extra_content:
    for extra in args.extra_content:
        if os.path.exists(extra):
            copy_recursive(extra, contentDir)

restrictedOuterFiles = []
for file in RESTRICTED_FILES:
    restrictedOuterFiles.append(os.path.join(packSource, file))

copy_recursive(packSource, packageDir, restrictedOuterFiles, True)

encryptListFile = os.path.join(packSource, ENCRYPT_FILES_FILE)
if os.path.exists(encryptListFile):
    filesToEncrypt = read_lines(encryptListFile)
    for file in filesToEncrypt:
        fullFilename = os.path.join(packageDir, file)
        if os.path.exists(fullFilename):
            if verbosity > 0:
                print("Encrypting file %s..." % (file))
            subprocess.call(["openssl", "enc", "-aes-256-cbc", "-in", fullFilename, "-out", fullFilename + ".enc", "-k", "OutdatedVersion", "-pbkdf2", "-iter", "100000", "-salt"],
                            stdout=None if verbosity < 2 else subprocess.DEVNULL, stderr=None if verbosity < 1 else subprocess.DEVNULL)
            os.remove(fullFilename)

if verbosity > 3:
    print("========== PACK CONTENTS ==========")
    subprocess.call(["ls", "-lh", packageDir])
    print("===================================")

if os.path.exists(packSaveFilename):
    os.remove(packSaveFilename)

ret = subprocess.call(["mksquashfs", packageDir, packSaveFilename, "-quiet"])

if ret == 0:
    print("Successfully created pack file and saved to %s" % (packSaveFilename))

shutil.rmtree(packageDir, True)

exit(0)
